@@ -1,10 +1,13 @@
 from django.contrib.contenttypes.models import ContentType
+from django.contrib.auth.models import User
 from django.conf import settings
-from django.utils import timezone
+from django.core.urlresolvers import reverse
 from django.utils.safestring import mark_safe
+from django.utils import timezone
 from django.db import models
 from django.db.models import Avg, Min, Max, Count, Sum
-
+from django.db.models.signals import post_save
+from report_builder.unique_slugify import unique_slugify
 from dateutil import parser
 
 class Report(models.Model):
@@ -20,10 +23,22 @@ class Report(models.Model):
     
     name = models.CharField(max_length=255)
     slug = models.SlugField(verbose_name="Short Name")
+    description = models.TextField(blank=True)
     root_model = models.ForeignKey(ContentType, limit_choices_to={'pk__in':_get_allowed_models})
     created = models.DateField(auto_now_add=True)
     modified = models.DateField(auto_now=True)
+    user_created = models.ForeignKey(User, editable=False, blank=True, null=True)
+    user_modified = models.ForeignKey(User, editable=False, blank=True, null=True, related_name="report_modified_set")
     distinct = models.BooleanField()
+    starred = models.ManyToManyField(User, blank=True,
+                                     help_text="These users have starred this report for easy reference.",
+                                     related_name="report_starred_set")
+    
+    
+    def save(self, *args, **kwargs):
+        if not self.id:
+            unique_slugify(self, self.name)
+        super(Report, self).save(*args, **kwargs)
 
 
     def add_aggregates(self, queryset):
@@ -109,7 +124,42 @@ class Report(models.Model):
     def get_absolute_url(self):
         return ("report_update_view", [str(self.id)])
     
+    def edit(self):
+        return mark_safe('<a href="%s"><img style="width: 26px; margin: -6px" src="/static/report_builder/img/edit.png"/></a>' % self.get_absolute_url())
+    edit.allow_tags = True
     
+    def download_xlsx(self):
+        return mark_safe('<a href="{0}"><img style="width: 26px; margin: -6px" src="/static/report_builder/img/download.svg"/></a>'.format(
+            reverse('report_builder.views.download_xlsx', args=[self.id])))
+    download_xlsx.short_description = "Download"
+    download_xlsx.allow_tags = True
+    
+
+    def copy_report(self):
+        return '<a href="{0}"><img style="width: 26px; margin: -6px" src="/static/report_builder/img/copy.svg"/></a>'.format(
+            reverse('report_builder.views.create_copy', args=[self.id]))
+    copy_report.short_description = "Copy"
+    copy_report.allow_tags = True
+
+    def check_report_display_field_positions(self):
+        """ After report is saved, make sure positions are sane
+        """
+        for i, display_field in enumerate(self.displayfield_set.all()):
+            if display_field.position != i+1:
+                display_field.position = i+1
+                display_field.save()
+
+
+class Format(models.Model):
+    """ A specifies a Python string format for e.g. `DisplayField`s. 
+    """
+    name = models.CharField(max_length=50, blank=True, default='')
+    string = models.CharField(max_length=300, blank=True, default='', help_text='Python string format. Ex ${} would place a $ in front of the result.')
+
+    def __unicode__(self):
+        return self.name
+    
+
 class DisplayField(models.Model):
     """ A display field to show in a report. Always belongs to a Report
     """
@@ -136,6 +186,7 @@ class DisplayField(models.Model):
     position = models.PositiveSmallIntegerField(blank = True, null = True)
     total = models.BooleanField(default=False)
     group = models.BooleanField(default=False)
+    display_format = models.ForeignKey(Format, blank=True, null=True)
 
     class Meta:
         ordering = ['position']
