@@ -1,9 +1,37 @@
+// place this in your static files folder, in:
+//    <static>/report_builder/js/report_form.js
+
+// initialize path prefix.
+path_prefix = ""
+
+// get current path
+current_path =  window.location.pathname;
+
+// where is "/report_builder"?
+root_index = current_path.indexOf( "/report_builder" );
+
+// other than 0?
+if ( root_index > 0 )
+{
+
+    // yes.  Get all text from start to that location, store it as path_prefix.
+    path_prefix = current_path.substring( 0, root_index );
+
+}
+else
+{
+
+    // no - set path_prefix to "".
+    path_prefix = "";
+
+}
+
 function expand_related(event, model, field, path, path_verbose) {
     if (event.target.tagName != 'LI') return;
     var element = $(event.target);
     if ( $(element).hasClass('tree_closed') ){
         $.get(  
-            "/report_builder/ajax_get_related/",  
+            path_prefix + "/report_builder/ajax_get_related/",  
             {model: model, field: field, path: path, path_verbose: path_verbose},
             function(data){
                 $(element).addClass('tree_expanded');
@@ -22,7 +50,7 @@ function show_fields(event, model, field, path, path_verbose){
     $('.highlight').removeClass('highlight');
     $(event.target).addClass('highlight');
     $.get(  
-        "/report_builder/ajax_get_fields/",  
+        path_prefix + "/report_builder/ajax_get_fields/",  
         {model: model, field: field, path: path, path_verbose: path_verbose},
         function(data){
             $('#field_selection_div').html(data);
@@ -54,8 +82,9 @@ function enable_drag() {
             total_forms.val(parseInt(i)+1);
             
             row_html = '<tr><td><span style="cursor: move;" class="ui-icon ui-icon-arrowthick-2-n-s"></span></td>';
-            row_html += '<td><input id="id_displayfield_set-'+i+'-path_verbose" name="displayfield_set-'+i+'-path_verbose" readonly="readonly" type="text" value="' + path_verbose + '"/></td>';
-            row_html += '<td><input id="id_displayfield_set-'+i+'-field_verbose" name="displayfield_set-'+i+'-field_verbose" readonly="readonly" type="text" value="' + field + '"/><input id="id_displayfield_set-'+i+'-path" name="displayfield_set-'+i+'-path" type="hidden" value="' + path + '"/></td>';
+            row_html += '<td><input type="checkbox" name="displayfield_set-'+i+'-DELETE" id="id_displayfield_set-'+i+'-DELETE">';
+            row_html += '<td>'+field+'<input id="id_displayfield_set-'+i+'-path_verbose" name="displayfield_set-'+i+'-path_verbose" readonly="readonly" type="hidden" value="' + path_verbose + '"/>';
+            row_html += '<input id="id_displayfield_set-'+i+'-field_verbose" name="displayfield_set-'+i+'-field_verbose" readonly="readonly" type="hidden" value="' + field + '"/><input id="id_displayfield_set-'+i+'-path" name="displayfield_set-'+i+'-path" type="hidden" value="' + path + '"/></td>';
             row_html += '<td><input id="id_displayfield_set-'+i+'-field" name="displayfield_set-'+i+'-field" type="hidden" value="' + label + '"/>'
             row_html += '<input id="id_displayfield_set-'+i+'-name" name="displayfield_set-'+i+'-name" type="text" value="' + name + '"/></td>';
             row_html += '<td><input type="text" name="displayfield_set-'+i+'-sort" class="small_input" id="id_displayfield_set-'+i+'-sort">';
@@ -66,18 +95,22 @@ function enable_drag() {
             } else {
                 row_html += '<td></td>'
             }
+            row_html += '<td><select id="id_displayfield_set-'+i+'-display_format" name="displayfield_set-'+i+'-display_format"> /* populate options from DB on drop */ </select></td>';
             row_html += '<td><input type="checkbox" class="small_input" name="displayfield_set-'+i+'-total" id="id_displayfield_set-'+i+'-total"></td>';
-            row_html += '<td><input type="checkbox" name="displayfield_set-'+i+'-group" id="id_displayfield_set-'+i+'-group"></td>';
-            row_html += '<td><input type="checkbox" name="displayfield_set-'+i+'-DELETE" id="id_displayfield_set-'+i+'-DELETE">';
+            row_html += '<td><input type="checkbox" name="displayfield_set-'+i+'-group" id="id_displayfield_set-'+i+'-group">';
             row_html += '<span class="hide_me"><input type="text" name="displayfield_set-'+i+'-position" value="'+total_forms.val()+'" id="id_displayfield_set-'+i+'-position"></span></td>';
             row_html += '</tr>';
             $('#field_list_table > tbody:last').append(row_html);
+
+            get_formats($('#'+'id_displayfield_set-'+i+'-display_format'));
+
         }
+
     });
 
 
     $("span.button[data-choices='true']").mousedown(function() {
-        $.get('/report_builder/ajax_get_choices/', {
+        $.get( path_prefix + '/report_builder/ajax_get_choices/', {
             'path_verbose': $(this).data('path_verbose'),
             'label': $(this).data('label'),
             'root_model': $(this).data('root_model'),
@@ -89,6 +122,11 @@ function enable_drag() {
     });
 
 
+    function get_formats(select) { 
+        $(select).load( path_prefix + '/report_builder/ajax_get_formats/' );
+    }
+
+
     $("#field_filter_droppable" ).droppable({
         drop: function( event, ui ) {
             field = $.trim($(ui.draggable).text());
@@ -111,7 +149,9 @@ function enable_drag() {
             
             row_html = '<tr>'
             row_html += '<td><span style="cursor: move;" class="ui-icon ui-icon-arrowthick-2-n-s"></span></td>'
-            row_html += '<td><input id="id_fil-'+i+'-path_verbose" value="'+ path_verbose +'" readonly="readonly" type="text" name="fil-'+i+'-path_verbose" maxlength="2000"></td>'
+            row_html += '<td><input type="checkbox" name="fil-'+i+'-DELETE" id="id_fil-'+i+'-DELETE">'
+            row_html += '<span class="hide_me"><input type="text" name="fil-'+i+'-position" value="0" id="id_fil-'+i+'-position"></span>'
+            row_html += '<input id="id_fil-'+i+'-path_verbose" value="'+ path_verbose +'" readonly="readonly" type="hidden" name="fil-'+i+'-path_verbose" maxlength="2000"></td>'
             row_html += '<td><input type="hidden" name="fil-'+i+'-field" value="'+ label +'" id="id_fil-'+i+'-field">'
             row_html += '<input name="fil-'+i+'-field_verbose" value="'+ field +'" readonly="readonly" maxlength="2000" type="text" id="id_fil-'+i+'-field_verbose">'
             row_html += '<input type="hidden" value="'+ path +'" name="fil-'+i+'-path" id="id_fil-'+i+'-path"></td>'
@@ -144,8 +184,6 @@ function enable_drag() {
                 row_html += '<td><input id="id_fil-'+i+'-filter_value" type="text" name="fil-'+i+'-filter_value" value="" maxlength="2000"></td>'
             }
             row_html += '<td><input type="checkbox" name="fil-'+i+'-exclude" id="id_fil-'+i+'-exclude"></td>'
-            row_html += '<td><input type="checkbox" name="fil-'+i+'-DELETE" id="id_fil-'+i+'-DELETE">'
-            row_html += '<span class="hide_me"><input type="text" name="fil-'+i+'-position" value="0" id="id_fil-'+i+'-position"></span></td>'
             row_html += '</tr>'
             $('#field_filter_table > tbody:last').append(row_html);
             $( ".datepicker" ).datepicker();
@@ -187,7 +225,7 @@ function set_check_value(event) {
 
 function refresh_preview() {
     $.post(  
-        "/report_builder/ajax_preview/",  
+        path_prefix + "/report_builder/ajax_preview/",  
         {
             csrfmiddlewaretoken: $("input[name=csrfmiddlewaretoken]").val(),
             report_id: $('#report_id').data('id'),
@@ -250,3 +288,17 @@ $(function() {
     });
     window.onbeforeunload = "Are you sure you want to leave?";
 });
+
+function filter(element, list) {
+    var value = $(element).val();
+
+    $("#"+list+" li").each(function() {
+        if ($(this).text().toUpperCase().search(value.toUpperCase()) > -1) {
+            $(this).show();
+        }
+        else {
+            $(this).hide();
+        }
+    });
+}
+
