@@ -3,38 +3,42 @@ from django.core import exceptions
 from django.conf import settings
 from django.contrib.admin.views.decorators import staff_member_required
 from django.contrib.auth.decorators import permission_required
+from django.contrib.contenttypes.models import ContentType
+from django.db.models.fields.related import ReverseManyRelatedObjectsDescriptor
 from django.forms.models import inlineformset_factory
 from django.http import HttpResponse, HttpResponseRedirect
-from django.shortcuts import render_to_response, get_object_or_404
+from django.shortcuts import render_to_response, redirect, get_object_or_404, render
 from django.template import RequestContext
-from report_builder.models import Report, DisplayField, FilterField
+from report_builder.models import Report, DisplayField, FilterField, Format
+from report_builder.utils import javascript_date_format, duplicate
 from django.utils.decorators import method_decorator
-from django.views.generic.edit import CreateView
-from django.views.generic.edit import UpdateView
+from django.views.generic.edit import CreateView, UpdateView
+from django.views.generic import ListView
 from django import forms
 
 import datetime
 import time
 import re
-from operator import itemgetter
 from decimal import Decimal
 from numbers import Number
 from types import BooleanType
-
+import copy
 from dateutil import parser
 
 
 class ReportForm(forms.ModelForm):
     class Meta:
         model = Report
-        fields = ['name', 'distinct', 'root_model', 'slug']
+        fields = ['name', 'distinct', 'root_model']
 
 
 class ReportEditForm(forms.ModelForm):
     class Meta:
         model = Report
-        fields = ['name', 'distinct']
-    
+        fields = ['name', 'distinct', 'description',]
+        widgets = {
+            'description': forms.TextInput(attrs={'style': 'width:99%;'}),
+        }
     
 class DisplayFieldForm(forms.ModelForm):
     class Meta:
@@ -66,6 +70,10 @@ class FilterFieldForm(forms.ModelForm):
         # override the filter_value field with the models native ChoiceField
         if self.instance.choices:
             self.fields['filter_value'].widget = forms.Select(choices=self.instance.choices)
+        if 'DateField' in self.instance.field_verbose or 'DateTimeField' in self.instance.field_verbose:
+            widget = self.fields['filter_value'].widget
+            widget.attrs['class'] = 'datepicker'
+            widget.attrs['data-date-format'] = javascript_date_format(settings.DATE_FORMAT)
 
 
 class ReportCreateView(CreateView):
@@ -224,14 +232,8 @@ def ajax_get_fields(request):
     model = ContentType.objects.get(pk=request.GET['model']).model_class()
     path = request.GET['path']
     path_verbose = request.GET.get('path_verbose')
-    if path:
-        # If there is a path, properties are not allowed as it will
-        # break things. See issues #10 and #11
-        properties = None
-        custom_fields = None
-    else:
-        properties = get_properties_from_model(model)
-        custom_fields = get_custom_fields_from_model(model)
+    properties = get_properties_from_model(model)
+    custom_fields = get_custom_fields_from_model(model)
     root_model = model.__name__.lower()
 
     if field_name == '':
@@ -264,10 +266,9 @@ def ajax_get_fields(request):
         path_verbose = new_model.__name__.lower()
    
     fields = get_direct_fields_from_model(new_model)
-    if not path:
-        custom_fields = get_custom_fields_from_model(new_model)
-        properties = get_properties_from_model(new_model)
-    
+    custom_fields = get_custom_fields_from_model(new_model)
+    properties = get_properties_from_model(new_model)
+
     return render_to_response('report_builder/report_form_fields_li.html', {
         'fields': fields,
         'custom_fields': custom_fields,
@@ -282,8 +283,14 @@ def ajax_get_choices(request):
     label = request.GET.get('label')
     root_model = request.GET.get('root_model')
     choices = FilterField().get_choices(path_verbose or root_model, label)
-    select_widget = forms.Select(choices=choices)
-    options_html = select_widget.render_options(select_widget.choices, [0])
+    select_widget = forms.Select(choices=[('','---------')] + list(choices))
+    options_html = select_widget.render_options([], [0])
+    return HttpResponse(options_html)
+
+def ajax_get_formats(request):
+    choices = Format.objects.values_list('pk', 'name')
+    select_widget = forms.Select(choices=[('','---------')] + list(choices))
+    options_html = select_widget.render_options([], [0])
     return HttpResponse(options_html)
 
 def get_model_from_path_string(root_model, path):
@@ -301,13 +308,31 @@ def get_model_from_path_string(root_model, path):
     return root_model
 
 
-def report_to_list(report, user, preview=False):
+def sort_helper(x, sort_key):
+    # TODO: explain what's going on here - I think this is meant to deal with
+    # null comparisons for datetimes? 
+    if x[sort_key] == None:
+        result = datetime.date(datetime.MINYEAR, 1, 1)
+    else:
+        result = x[sort_key]     
+    return result.lower() if isinstance(result, basestring) else result
+
+def report_to_list(report, user, preview=False, queryset=None):
     """ Create list from a report with all data filtering
+    preview: Return only first 50
+    objects: Provide objects for list, instead of running filters
     Returns list, message in case of issues
     """
     message= ""
     model_class = report.root_model.model_class()
-    objects = report.get_query()
+    if queryset != None:
+        objects = report.add_aggregates(queryset)
+    else:
+        try:
+            objects = report.get_query()
+        except exceptions.ValidationError, e:
+            message += "Validation Error: {0!s}. This probably means something is wrong with the report's filters.".format(e)
+            return [], message
 
     # Display Values
     display_field_paths = []
@@ -316,8 +341,9 @@ def report_to_list(report, user, preview=False):
     display_totals = {}
     def append_display_total(display_totals, display_field, display_field_key):
         if display_field.total:
-            display_totals[display_field_key] = {'label': display_field.name, 'val': Decimal('0.00')}
-        
+            display_totals[display_field_key] = {'val': Decimal('0.00')}
+
+
     for i, display_field in enumerate(report.displayfield_set.all()):
         model = get_model_from_path_string(model_class, display_field.path)
         if user.has_perm(model._meta.app_label + '.change_' + model._meta.module_name) \
@@ -360,10 +386,6 @@ def report_to_list(report, user, preview=False):
         if user.has_perm(report.root_model.app_label + '.change_' + report.root_model.model) \
         or user.has_perm(report.root_model.app_label + '.view_' + report.root_model.model):
 
-            property_filters = {} 
-            for property_filter in report.filterfield_set.filter(field_verbose__contains='[property]'):
-                property_filters[property_filter.field] = property_filter 
-
             def increment_total(display_field_key, display_totals, val):
                 if display_totals.has_key(display_field_key):
                     # Booleans are Numbers - blah
@@ -373,14 +395,24 @@ def report_to_list(report, user, preview=False):
                     else:
                         display_totals[display_field_key]['val'] += Decimal('1.00')
 
-            # get pk in order to retrieve object for adding properties to report rows
+
+            # get pk for primary and m2m relations in order to retrieve objects 
+            # for adding properties to report rows
             display_field_paths.insert(0, 'pk')
+            m2m_relations = []
+            for position, property_path in property_list.iteritems():
+                property_root = property_path.split('__')[0]
+                root_class = report.root_model.model_class()
+                property_root_class = getattr(root_class, property_root)
+                if type(property_root_class) == ReverseManyRelatedObjectsDescriptor:
+                    display_field_paths.insert(1, '%s__pk' % property_root)
+                    m2m_relations.append(property_root)
             values_and_properties_list = []
             filtered_report_rows = []
             group = None 
             for df in report.displayfield_set.all():
                 if df.group:
-                    group = df.field
+                    group = df.path + df.field
                     break
             if group:
                 filtered_report_rows = report.add_aggregates(objects.values_list(group))
@@ -391,11 +423,25 @@ def report_to_list(report, user, preview=False):
                 for row in values_list:
                     row = list(row)
                     obj = report.root_model.model_class().objects.get(pk=row.pop(0)) 
+                    #related_objects
                     remove_row = False
                     values_and_properties_list.append(row)
                     # filter properties (remove rows with excluded properties)
-                    for property_filter_label, property_filter in property_filters.iteritems():
-                        val = reduce(getattr, (property_filter.path + property_filter.field).split('__'), obj)
+                    property_filters = report.filterfield_set.filter(
+                        field_verbose__contains='[property]'
+                        )
+                    for property_filter in property_filters: 
+                        root_relation = property_filter.path.split('__')[0]
+                        if root_relation in m2m_relations: 
+                            pk = row[0]
+                            if pk is not None:
+                                # a related object exists
+                                m2m_obj = getattr(obj, root_relation).get(pk=pk)
+                                val = reduce(getattr, [property_filter.field], m2m_obj)
+                            else:
+                                val = None
+                        else:
+                            val = reduce(getattr, (property_filter.path + property_filter.field).split('__'), obj)
                         if filter_property(property_filter, val):
                             remove_row = True
                             values_and_properties_list.pop()
@@ -406,7 +452,18 @@ def report_to_list(report, user, preview=False):
                             if field in display_totals.keys():
                                 increment_total(field, display_totals, row[i])
                         for position, display_property in property_list.iteritems(): 
-                            val = reduce(getattr, display_property.split('__'), obj)
+                            relations = display_property.split('__')
+                            root_relation = relations[0]
+                            if root_relation in m2m_relations: 
+                                pk = row.pop(0)
+                                if pk is not None:
+                                    # a related object exists
+                                    m2m_obj = getattr(obj, root_relation).get(pk=pk)
+                                    val = reduce(getattr, relations[1:], m2m_obj)
+                                else:
+                                    val = None
+                            else:
+                                val = reduce(getattr, relations, obj)
                             values_and_properties_list[-1].insert(position, val)
                             increment_total(display_property, display_totals, val)
                         for position, display_custom in custom_list.iteritems(): 
@@ -416,20 +473,23 @@ def report_to_list(report, user, preview=False):
                         filtered_report_rows += [values_and_properties_list[-1]]
                     if preview and len(filtered_report_rows) == 50:
                         break
-            sort_fields = report.displayfield_set.filter(sort__gt=0).order_by('sort').\
-                values_list('position', flat=True)
-            if sort_fields:
-                get_key = itemgetter(*[s-1 for s in sort_fields])
-                values_and_properties_list = sorted(
-                    filtered_report_rows,
-                    key=lambda x: get_key(x).lower() if isinstance(get_key(x), basestring) else get_key(x)
-                )
+            sort_fields = report.displayfield_set.filter(sort__gt=0).order_by('-sort').\
+                values_list('position', 'sort_reverse')
+            for sort_field in sort_fields:
+                filtered_report_rows = sorted(
+                        filtered_report_rows,
+                        key=lambda x: sort_helper(x, sort_field[0]-1),
+                        reverse=sort_field[1]
+                        )
+            values_and_properties_list = filtered_report_rows
         else:
             values_and_properties_list = []
             message = "Permission Denied on %s" % report.root_model.name
 
-        # add choice list display
+
+        # add choice list display and display field formatting
         choice_lists = {} 
+        display_formats = {} 
         final_list = []
         for df in report.displayfield_set.all():
             if df.choices:
@@ -438,32 +498,64 @@ def report_to_list(report, user, preview=False):
                 df_choices[''] = ''
                 df_choices[None] = ''
                 choice_lists.update({df.position: df_choices}) 
-        if choice_lists:
-            for row in values_and_properties_list:
-                row = list(row)
-                for position, choice_list in choice_lists.iteritems():
-                    row[position-1] = choice_list[row[position-1]]
-                    final_list.append(row)
-            values_and_properties_list = final_list
-
-        # add display totals for grouped result sets
-        if group:
-            # increment totals for fields
-            for row in values_and_properties_list: 
+            if df.display_format:
+                display_formats.update({df.position: df.display_format}) 
+        for row in values_and_properties_list:
+            # add display totals for grouped result sets
+            # TODO: dry this up, duplicated logic in non-grouped total routine 
+            if group:
+                # increment totals for fields
                 for i, field in enumerate(display_field_paths[1:]):
                     if field in display_totals.keys():
                         increment_total(field, display_totals, row[i])
+            row = list(row)
+            for position, choice_list in choice_lists.iteritems():
+                row[position-1] = choice_list[row[position-1]]
+            for position, display_format in display_formats.iteritems():
+                # convert value to be formatted into Decimal in order to apply
+                # numeric formats
+                try:
+                    value = Decimal(row[position-1])
+                except:
+                    value = row[position-1]
+                # Try to format the value, let it go without formatting for ValueErrors
+                try:
+                    row[position-1] = display_format.string.format(value)
+                except ValueError:
+                    row[position-1] = value
+            final_list.append(row)
+        values_and_properties_list = final_list
+
 
         if display_totals:
-            display_totals_row = ['TOTALS'] + [
-                '%s: %s' % (
-                    display_totals[t]['label'],
-                    display_totals[t]['val']
-                ) for t in display_totals
-            ]
+            display_totals_row = []
+            
+            fields_and_properties = list(display_field_paths[1:])
+            for position, value in property_list.iteritems(): 
+                fields_and_properties.insert(position, value)
+            for i, field in enumerate(fields_and_properties): 
+                if field in display_totals.keys():
+                    display_totals_row += [display_totals[field]['val']]
+                else:
+                    display_totals_row += ['']
+
+            # add formatting to display totals
+            for df in report.displayfield_set.all():
+                if df.display_format:
+                    try:
+                        value = Decimal(display_totals_row[df.position-1])
+                    except:
+                        value = display_totals_row[df.position-1]
+                    display_totals_row[df.position-1] = df.display_format.string.\
+                        format(value)
 
         if display_totals:
-            values_and_properties_list = values_and_properties_list + [display_totals_row]
+            values_and_properties_list = (
+                values_and_properties_list + [
+                    ['TOTALS'] + (len(fields_and_properties) - 1) * ['']
+                    ] + [display_totals_row]
+                )
+
                 
 
     except exceptions.FieldError:
@@ -551,14 +643,17 @@ class ReportUpdateView(UpdateView):
             field_list_formset.save()
             field_filter_formset.report = self.object
             field_filter_formset.save()
+            self.object.check_report_display_field_positions()
             return HttpResponseRedirect(self.get_success_url())
         else:
             return self.render_to_response(self.get_context_data(form=form))
         
 @staff_member_required
-def download_xlsx(request, pk):
+def download_xlsx(request, pk, queryset=None):
     """ Download the full report in xlsx format
-    Why xlsx? Because there is no decent ods library for python and xls has limitations """
+    Why xlsx? Because there is no decent ods library for python and xls has limitations 
+    queryset: predefined queryset to bypass filters
+    """
     import cStringIO as StringIO
     from openpyxl.workbook import Workbook
     from openpyxl.writer.excel import save_virtual_workbook
@@ -580,9 +675,14 @@ def download_xlsx(request, pk):
         ws.column_dimensions[get_column_letter(i+1)].width = field.width
         i += 1
     
-    objects_list, message = report_to_list(report, request.user)
+    objects_list, message = report_to_list(report, request.user, queryset=queryset)
     for row in objects_list:
-        ws.append(row)
+        try:
+            ws.append(row)
+        except ValueError as e:
+            ws.append([e.message])
+        except:
+            ws.append(['Unknown Error'])
     
     myfile = StringIO.StringIO()
     myfile.write(save_virtual_workbook(wb))
@@ -593,5 +693,65 @@ def download_xlsx(request, pk):
     response['Content-Length'] = myfile.tell()
     return response
     
+
+@staff_member_required
+def ajax_add_star(request, pk):
+    """ Star or unstar report for user
+    """
+    report = get_object_or_404(Report, pk=pk)
+    user = request.user
+    if user in report.starred.all():
+        added = False
+        report.starred.remove(request.user)
+    else:
+        added = True
+        report.starred.add(request.user)
+    return HttpResponse(added)
     
+@staff_member_required
+def create_copy(request, pk):
+    """ Copy a report including related fields """
+    report = get_object_or_404(Report, pk=pk)
+    new_report = duplicate(report, changes=(
+        ('name', '{} (copy)'.format(report.name)),
+        ('user_created', request.user),
+        ('user_modified', request.user),
+    ))
+    # duplicate does not get related
+    for display in report.displayfield_set.all():
+        new_display = copy.copy(display)
+        new_display.pk = None
+        new_display.report = new_report
+        new_display.save()
+    for report_filter in report.filterfield_set.all():
+        new_filter = copy.copy(report_filter)
+        new_filter.pk = None
+        new_filter.report = new_report
+        new_filter.save()
+    return redirect(new_report)
+
+
+@staff_member_required
+def export_to_report(request):
+    """ Export objects (by ID and content type) to an existing or new report
+    In effect this runs the report with it's display fields. It ignores 
+    filters and filters instead the provided ID's. It can be select
+    as a global admin action.
+    """
+    admin_url = request.GET.get('admin_url', '/')
+    ct = ContentType.objects.get_for_id(request.GET['ct'])
+    ids = request.GET['ids'].split(',')
+    number_objects = len(ids)
+    reports = Report.objects.filter(root_model=ct).order_by('-modified')
     
+    if 'download' in request.GET:
+        report = get_object_or_404(Report, pk=request.GET['download'])
+        queryset = ct.model_class().objects.filter(pk__in=ids)
+        return download_xlsx(request, report.id, queryset=queryset)
+    
+    return render(request, 'report_builder/export_to_report.html', {
+        'object_list': reports,
+        'admin_url': admin_url,
+        'number_objects': number_objects,
+        'model': ct.model_class()._meta.verbose_name,
+        })
